#! /usr/bin/env python

import traceback
import logging

# process a node and assemble the info to build a scene,
# currently only processes scenes with a flag of 132

logger = logging.getLogger(__name__)


class Scene_Info(object):
    def __init__(self, group):

        self.valid = False

        try:

            self.flag = group.attrib["flag"]
            self.id = group.attrib["nodeDefId"]

            address = group.find("address")
            self.address = address.text

            name = group.find("name")
            self.name = name.text

            self.family = group.find("family").text

            if self.flag == "12":  # ignore root scene
                logger.info("ignoring root scene: {}".format(self.name))
                return

            self.primary_node = group.find("pnode").text

            self.device_group = group.find("deviceGroup").text

            self.controllers = []  # list of controller addresses
            self.responders = []  # list of responder addresses

            members = group.find("members")
            for link in members.iterfind("link"):
                link_type = link.attrib["type"]
                if link_type == "0":  # responder
                    self.responders.append(link.text)
                elif link_type == "16":  # controller
                    self.controllers.append(link.text)

            self.valid = True

        except Exception as ex:
            # print ('scene info error',ex)
            traceback.print_exc()

    def __repr__(self):
        return "Scene: Name {} Address {}, Family {}, Flag {}, Controllers {}, Responders {}".format(
            self.name,
            self.address,
            self.family,
            self.flag,
            self.controllers,
            self.responders,
        )



#! /usr/bin/env python


class Variable_Info(object):
    def __init__(self, node):

        self.valid = False

        try:

            self.id = node.attrib["id"]
            self.type = node.attrib["type"]
            self.init_value = int(node.find("init").text)
            self.value = int(node.find("val").text)
            self.time_set = node.find("ts").text

            self.valid = True

        except:
            pass

    def __repr__(self):
        return "Variable: ID {} Value {}, Init {}, Time {}".format(
            self.id, self.value, self.init_value, self.time_set
        )




#! /usr/bin/env python


import xml.etree.ElementTree as ET
import traceback

from ..item_container import Item_Container
from .scene_info import Scene_Info
from .scene_insteon import Scene_Insteon

import logging

logger = logging.getLogger(__name__)


scene_classes = {
    "6": Scene_Insteon,
}


class Scene_Container(Item_Container):
    def __init__(self, controller):
        Item_Container.__init__(self, controller, "Scene")

    def start(self):
        success, response = self.controller.send_request("nodes/scenes")

        if success:
            try:
                root = ET.fromstring(response)
                self.process_scene_nodes(root)
                self.items_retrieved = True
                return True

            except Exception as ex:
                logger.error("container manager Error {}".format(ex))
                traceback.print_exc()
        else:
            return False

    def process_scene_nodes(self, root):
        for scene in root.iter("group"):
            self.process_scene_node(scene)

    def process_scene_node(self, node):
        scene_info = Scene_Info(node)

        if scene_info.valid:  # make sure we have the info we need
            # print('process scene',scene_info)
            if scene_info.family in scene_classes:
                scene_class = scene_classes[scene_info.family]

                scene = scene_class(self, scene_info)
                scene.update_onoff()
                self.add(scene, scene.address)
        else:
            logger.warn("Invalid scene info {}".format(scene_info.name))

    def device_event(
        self, device
    ):  # notification from controller about a device event, used to "track" scene state
        for address, scene in self.list.items():
            scene.device_event(device)

    def get_device(self, address):
        return self.controller.device_container.get(address)




import zeep


from requests import Session
from requests.auth import HTTPBasicAuth  # or HTTPDigestAuth, or OAuth1, etc.
from zeep import Client
from zeep.transports import Transport

session = Session()
session.auth = HTTPBasicAuth('admin','admin')
client = Client('http://192.168.1.51/services.wsdl',
    transport=Transport(session=session))

client.settings.strict=False
print (client.wsdl.services.values())
print(client.service.GetStartupTime())
print(client.service.GetSceneProfiles())
print(client.service.GetSystemDateTime())
print(client.service.GetSystemStatus())
#print(client.service.GetISYConfig())
print(client.service.GetNodesConfig())
print(client.service.GetStartupTime())


'''
for service in client.wsdl.services.values():
    print ("service:", service.name)
    for port in service.ports.values():
        operations = sorted(
            port.binding._operations.values(),
            key=operator.attrgetter('name'))

        for operation in operations:
            print ("method :", operation.name)
            print ("  input :", operation.input.signature())
            print ("  output:", operation.output.signature())
            print()
    print()


'''

#print (client.wsdl.services.values())
print(client.service.GetSceneProfiles())


#! /usr/bin/env python

""" 

Base Variable

Common to all elements returned from rest/nodes/variables

   properties = list of properties and values

"""

from ..item_base import Item_Base


class Variable_Base(Item_Base):
    def __init__(self, container, variable_info):
        Item_Base.__init__(
            self, container, variable_info.id
        )  # replaced once names are retrieved

        self.id = variable_info.id
        self.type = variable_info.type

        self.add_property("init_value", variable_info.init_value)
        self.add_property("value", variable_info.value)
        self.add_property("time_set", variable_info.time_set)

    def get_index(self):  # creates an index using var type and id
        return self.type + ":" + self.id

    def __str__(self):
        return "Variable: {} ; index {} ; value {}".format(
            self.name, self.get_index(), self.get_property("value")
        )

    def process_websocket_event(self, event):
        var_node = event.event_info_node.find("var")
        value = var_node.find("val").text
        time_set = var_node.find("ts").text

        self.set_property("value", value)
        self.set_property("time_set", time_set)

    def set_value(self, value):
        path = "vars/set/" + self.type + "/" + self.id + "/" + str(value)
        return self.send_request(path)

    def get_identifier(self):
        return self.get_index()



#! /usr/bin/env python


from .device_base import Device_Base


class Device_Dimmer(Device_Base):
    def __init__(self, container, name, address):
        Device_Base.__init__(self, container, "dimmer", name, address)

        self.add_property("level", 0)  # in percent

    def set_level(self, level):
        pass




#! /usr/bin/env python


from .variable_base import Variable_Base


class Variable_Integer(Variable_Base):
    def __init__(self, container, variable_info):
        Variable_Base.__init__(self, container, variable_info)




#! /usr/bin/env python


from ..common.device_fan import Device_Fan
from .device_insteon_base import Device_Insteon_Base

speeds = ["off", "low", "medium", "high"]


class Device_Insteon_Fan(Device_Fan, Device_Insteon_Base):
    def __init__(self, container, device_info):
        Device_Fan.__init__(
            self, container, device_info.name, device_info.address, speeds
        )
        Device_Insteon_Base.__init__(self, device_info)

        value = device_info.get_property("ST", "value")
        if value:
            self.set_property("speed", self.level_to_speed(int(value)))

    def process_websocket_event(self, event):
        if event.control == "ST":
            self.set_property("speed", self.level_to_speed(int(event.action)))

    def set_speed(self, speed):
        level = self.speed_to_level(speed)
        path = "nodes/" + self.address + "/cmd/DON/" + str(level)
        return self.send_request(path)

    def level_to_speed(self, level):
        if level == 0:
            return "off"
        elif level > 0 and level <= 63:
            return "low"
        elif level >= 50 and level <= 191:
            return "medium"
        else:
            return "high"

    def speed_to_level(self, speed):
        if speed == "off":
            return 0
        elif speed == "low":
            return 63
        elif speed == "medium":
            return 99
        else:
            return 255




#! /usr/bin/env python

import xml.etree.ElementTree as ET
import traceback

from ..item_container import Item_Container
from .device_info import Device_Info
from .insteon.insteon_device_manager import get_insteon_device_class
from .zwave.zwave_device_manager import get_zwave_device_class

import logging

logger = logging.getLogger(__name__)


class Device_Container(Item_Container):
    def __init__(self, controller):
        Item_Container.__init__(self, controller, "Device")

    def start(self):
        success, response = self.send_request("nodes/devices")

        if success:  # and response.status == 200:
            try:
                root = ET.fromstring(response)
                self.process_device_nodes(root)
                self.items_retrieved = True
                return True

            except Exception as ex:
                logger.error("container error {}".format(ex))
                traceback.print_exc()
        else:
            return False

    def process_device_nodes(self, root):
        for device in root.iter("node"):
            self.process_device_node(device)

    def process_device_node(self, node):
        device_info = Device_Info(node)  # parse node XML
        # print('process node',device_info.valid,device_info)

        if device_info.valid:  # make sure we have the info we need
            """ device family
            	0 = Default (core driver implementation e.g. Insteon, UPB)
				1 = Insteon products
				2 = UPB Products
				3 = RCS Products
				4 = ZWave products
				5 = Auto DR (for groups)
				6 = Generic (for groups)
				7 = UDI Products
				8 = Brultech Products
				9 = NCD Products
            """
            # TBD add support for other device families
            if device_info.family == "1":  # insteon devices
                device_class = get_insteon_device_class(device_info)
                if device_class is not None:
                    device = device_class(self, device_info)
                    self.add(device, device.address)
                else:
                    logger.warn("No device class for {}".format(device_info))

            elif device_info.family == "4":  # z-wave devices
                device_class = get_zwave_device_class(device_info)
                if device_class is not None:
                    device = device_class(self, device_info)
                    self.add(device, device.address)
                else:
                    logger.warn("No device class for {}".format(device_info))

    def websocket_event(self, event):
        # print('Device event',event)
        device = self.get(event.address)
        if device is not None:
            device.process_websocket_event(event)




#! /usr/bin/env python


import xml.etree.ElementTree as ET
import traceback

from ..item_container import Item_Container
from .program_info import Program_Info
from .program_base import Program_Base


import logging

logger = logging.getLogger(__name__)


class Program_Container(Item_Container):
    def __init__(self, controller):
        Item_Container.__init__(self, controller, "Program")

    def start(self):
        success, response = self.controller.send_request("programs/?subfolders=true")

        if success:
            try:
                root = ET.fromstring(response)
                self.process_program_nodes(root)
                self.items_retrieved = True
                return True

            except Exception as ex:
                logger.error("container error {}".format(ex))
                traceback.print_exc()
        else:
            return False

    def process_program_nodes(self, root):
        for program in root.iter("program"):
            self.process_program_node(program)

    def process_program_node(self, node):
        program_info = Program_Info(node)  # parse node XML

        if program_info.valid:  # make sure we have the info we need
            program = Program_Base(self, program_info)
            self.add(program, program.id)

    def websocket_event(self, event):
        # print('Program event',event)
        id = event.event_info_node.find("id").text.zfill(4)

        status = None
        run_time = None
        finish_time = None

        if event.event_info_node.find("s") is not None:
            status = int(event.event_info_node.find("s").text)

        if event.event_info_node.find("r") is not None:
            run_time = event.event_info_node.find("r").text

        if event.event_info_node.find("f") is not None:
            finish_time = event.event_info_node.find("f").text

        if status and run_time and finish_time:
            program = self.get(id)
            if program:
                program.process_websocket_event(status, run_time, finish_time)
            else:
                logger.warning("Unable able to find program type id {}".format(id))




#! /usr/bin/env python


from .scene_base import Scene_Base


class Scene_Insteon(Scene_Base):
    def __init__(self, container, scene_info):
        Scene_Base.__init__(self, container, scene_info)

        self.add_property("onoff")

    def process_websocket_event(self, event):
        pass  # there are no events for scenes AFAICT

    def turn_on(self):
        path = "nodes/" + self.address + "/cmd/DON"
        return self.send_request(path)

    def turn_off(self):
        path = "nodes/" + self.address + "/cmd/DOF"
        return self.send_request(path)

    def device_event(
        self, device
    ):  # device event, process and see if we are interested
        if device.family == "1":  # insteon device
            if device.address in self.responders:  # this scene has this device
                self.update_onoff()

    def update_onoff(
        self,
    ):  # iterates through responder devices and decides if state is on or off
        scene_onoff = "off"
        for address in self.responders:
            device = self.container.get_device(address)
            if device is not None:
                if device.category == "1":  # insteon dimmer
                    if device.get_property("level") > 0:
                        scene_onoff = "on"
                if device.category == "2":  # insteon switch
                    if device.get_property("onoff") == "on":
                        scene_onoff = "on"

        if scene_onoff != self.properties["onoff"]:
            self.set_property("onoff", scene_onoff)



#! /usr/bin/env python


from ..common.device_lock import Device_Lock
from .device_zwave_base import Device_ZWave_Base


class Device_ZWave_Lock(Device_Lock, Device_ZWave_Base):
    def __init__(self, container, device_info):
        Device_Lock.__init__(self, container, device_info.name, device_info.address)
        Device_ZWave_Base.__init__(self, device_info)

        value = device_info.get_property("ST", "value")
        if value:
            try:
                if int(value) > 0:
                    self.set_property("state", "Locked")
                else:
                    self.set_property("state", "Unlocked")
            except:
                pass

    def process_websocket_event(self, event):

        if event.control == "ST":
            state_map = {0: "Unlocked", 100: "Locked"}
            self.set_property("state", state_map[int(event.action)])
        elif event.control == "BATLVL":
            self.set_property("batterylevel", int(event.action))
        elif event.control == "ERR":
            self.set_property("error", int(event.action))
        elif event.control == "USRNUM":
            self.set_property("usernumber", int(event.action))

    def lock(self):
        path = "nodes/" + self.address + "/cmd/SECMD/1"
        return self.send_request(path)

    def unlock(self):
        path = "nodes/" + self.address + "/cmd/SECMD/0"
        return self.send_request(path)



#! /usr/bin/env python

""" 

Base Device

Common to all elements returned from rest/nodes/devices

device_type = switch,dimmer,contact,...

having a common device types eliminates the need to know the underlying technology being used

"""


from ...item_base import Item_Base


class Device_Base(Item_Base):
    def __init__(self, container, device_type, name, address):
        Item_Base.__init__(self, container, name)

        self.device_type = device_type
        self.address = address

    def __str__(self):
        return "Device {}, type {}, ID {}".format(
            self.name, self.device_type, self.address
        )

    def get_identifier(self):
        return self.address




#! /usr/bin/env python


from ..common.device_base import Device_Base
from .device_insteon_base import Device_Insteon_Base

paddle_events = {"DON", "DOF", "DIM", "BRT", "DFON", "DFOF"}


class Device_Insteon_Controller(Device_Base, Device_Insteon_Base):
    def __init__(self, container, device_info):
        Device_Base.__init__(
            self, container, "controller", device_info.name, device_info.address
        )
        Device_Insteon_Base.__init__(self, device_info)

        self.add_property("paddle_action")

    def process_websocket_event(self, event):
        if event.control in paddle_events:
            self.set_property("paddle_action", event.control, True)




#! /usr/bin/env python


from .device_base import Device_Base


class Device_Contact(Device_Base):
    def __init__(self, container, name, address):
        Device_Base.__init__(self, container, "contact", name, address)

        self.add_property("contact", None)




#! /usr/bin/env python

""" 
Only contains the ISY controller item, use the container for consistency
"""
from ..item_base import Item_Base


class Controller_Base(Item_Base):
    def __init__(self, container, id):
        Item_Base.__init__(self, container, "ISY Controller")

        self.id = id
        self.name = id

        self.add_property("state", "idle")
        self.add_property("heartbeat", "none")
        self.add_property("websocket", "disconnnected")
        self.add_property("http", "disconnected")

    def __str__(self):
        return "Controller: {}; state {}; status {}".format(
            self.id, self.properties["state"], self.properties["status"]
        )

    def get_identifier(self):
        return self.id

    def get_controller_time(self):
        success, response = self.send_request("time")
        if success:
            pass
        return success, response



#! /usr/bin/env python


from ..common.device_contact import Device_Contact
from .device_insteon_base import Device_Insteon_Base


class Device_Insteon_Contact(Device_Contact, Device_Insteon_Base):
    def __init__(self, container, device_info):
        Device_Contact.__init__(self, container, device_info.name, device_info.address)
        Device_Insteon_Base.__init__(self, device_info)

        value = device_info.get_property("ST", "value")
        if value:
            try:
                if int(value) > 0:
                    self.set_property("contact", "open")
                else:
                    self.set_property("contact", "closed")
            except:
                pass

    def process_websocket_event(self, event):
        if event.control == "ST":
            if int(event.action) > 0:
                self.set_property("contact", "open")
            else:
                self.set_property("contact", "closed")




#! /usr/bin/env python

""" 

returns a device instance using node data from an zwave device, None if unable to create device

"""

from .device_zwave_controller import Device_ZWave_Controller
from .device_zwave_lock import Device_ZWave_Lock
from .device_zwave_genericrspctl import Device_ZWave_GenericRspCtl
from .device_zwave_power import Device_ZWave_Power


zwave_device_classes = {
    "111": Device_ZWave_Lock,
    # '118' : Device_ZWave_MultilevelSensor,
    "121": Device_ZWave_GenericRspCtl,
    # '140' : Device_ZWave_Thermostat,
    "143": Device_ZWave_Power,
    # '155' : Device_ZWave_MotionSensor,
    # '156' : Device_ZWave_SmokeSensor,
    # '157' : Device_ZWave_TamperAlarm,
    # '172' : Device_ZWave_IntrusionAlarm,
    # '173' : Device_ZWave_TamperCodeAlarm,
    # '185' : Device_ZWave_NotificationSensor,
}


def get_zwave_device_class(device_info):

    # print ('Z-Wave Device Class')
    # look for specific device device cat that need special handling
    print(device_info)
    # if device_info.category == '1' and device_info.sub_category == '46' and device_info.address_parts [3] == '2': # fanlinc motor
    #     return Device_Insteon_Fan

    # print (device_info.node_def_id.find('KeypadButton') , device_info.address_parts [3])
    # override device cat for keypadlinc dimmer buttons and change to switch type devices
    # if device_info.node_def_id.find('KeypadButton') == 0 and device_info.address_parts [3] != '1': # maybe use node flag
    #     device_info.category = '2'

    # find device class
    # check for specific devcat/subcat

    # print (device_info, device_info.category, device_info.devtype_cat)

    if device_info.devtype_cat in zwave_device_classes:
        device_class = zwave_device_classes[device_info.devtype_cat]
        return device_class



#! /usr/bin/env python

""" 

returns a device instance using node data from an insteon device, None if unable to create device

"""

from .device_insteon_contact import Device_Insteon_Contact
from .device_insteon_dimmer import Device_Insteon_Dimmer
from .device_insteon_switch import Device_Insteon_Switch
from .device_insteon_fan import Device_Insteon_Fan
from .device_insteon_controller import Device_Insteon_Controller
from .device_insteon_templinc import Device_Insteon_TempLinc
from .device_insteon_siren import Device_Insteon_Siren

insteon_device_classes = {
    "0": Device_Insteon_Controller,
    "1": Device_Insteon_Dimmer,
    "2": Device_Insteon_Switch,
    "14": Device_Insteon_Switch,
    "16": Device_Insteon_Contact,
}

"""
dev_cat_sub_cat = {
    '5' : {
        '10' : Device_Insteon_TempLinc,
    },
}
"""


def get_insteon_device_class(device_info):
    # print ('Insteon Device Class')
    # look for specific device dev/sub cat that need special handling
    # print(device_info)
    if (
        device_info.category == "1"
        and device_info.sub_category == "46"
        and device_info.address_parts[3] == "2"
    ):  # fanlinc motor
        return Device_Insteon_Fan

    if (
        device_info.category == "5"
        and device_info.sub_category == "10"
        and device_info.address_parts[3] == "1"
    ):  # temp linc (Wireless)
        return Device_Insteon_TempLinc

    if (
        device_info.category == "5"
        and device_info.sub_category == "11"
        and device_info.address_parts[3] == "1"
    ):  # temp linc (Wired)
        return Device_Insteon_TempLinc

    if (
        device_info.category == "7"
        and device_info.sub_category == "0"
        and device_info.address_parts[3] == "1"
    ):  # IOLinc sensor
        return Device_Insteon_Contact

    if (
        device_info.category == "7"
        and device_info.sub_category == "0"
        and device_info.address_parts[3] == "2"
    ):  # IOLinc relay
        return Device_Insteon_Switch

    if (
        device_info.category == "7"
        and device_info.sub_category == "30"
        and device_info.address_parts[3] == "1"
    ):  # IOLinc sensor
        return Device_Insteon_Siren

    # print (device_info.node_def_id.find('KeypadButton') , device_info.address_parts [3])
    # override device cat for keypadlinc dimmer buttons and change to switch type devices
    if (
        device_info.node_def_id.find("KeypadButton") == 0
        and device_info.address_parts[3] != "1"
    ):  # maybe use node flag
        device_info.category = "2"

    # find device class
    # check for specific devcat/subcat

    # print (device_info,device_info.category)

    if device_info.category in insteon_device_classes:
        device_class = insteon_device_classes[device_info.category]
        return device_class


"""        
    if device_info.category in dev_cat_sub_cat and device_info.sub_category in dev_cat_sub_cat [device_info.category]:
        device_class = dev_cat_sub_cat [device_info.category] [device_info.sub_category]
        return device_class
"""



#! /usr/bin/env python

""" 

Base Device

Common to all elements returned from rest/nodes/devices

   properties = list of properties and values

"""


from ..common.device_base import Device_Base


class Device_Insteon_Base(object):
    def __init__(self, device_info):
        self.family = device_info.family
        self.category = device_info.category
        self.sub_category = device_info.sub_category
        self.version = device_info.version
        self.res = device_info.reserved
        # self.address = device_info.address
        self.address_parts = device_info.address_parts
        self.flag = device_info.flag
        # self.container_node_address = device_info.container_node_address

    @property
    def flag(self):
        return self._flag

    @flag.setter
    def flag(self, flag):
        self._flag = flag
        if flag & 16:
            self.set_property("status", "alert")# pylint: disable=no-member
        else:
            self.set_property("status", "ready")# pylint: disable=no-member

    def beep(self): #not all insteon devices
        path = "nodes/" + self.address + "/cmd/BEEP"# pylint: disable=no-member
        return self.send_request(path)# pylint: disable=no-member

    def query(self): #no value returns, generates a websocket event if value is changed
        path = "nodes/" + self.address + "/cmd/QUERY"# pylint: disable=no-member
        return self.send_request(path)# pylint: disable=no-member        